/*
 * Priscila: A GNU/EDMA OpenGL Presentation Tool
 * Copyright (C) 2004  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#ifndef THREE_DS_H
#define THREE_DS_H

#include "sdlgl_tex.h"

typedef struct color_t
{
  unsigned char r;
  unsigned char g;
  unsigned char b;
} COLOR;

typedef struct vector3D_t
{
  float   x;
  float   y;
  float   z;
} VECTOR3D;

typedef struct Vector3D_int_t
{
  unsigned short    a;
  unsigned short    b;
  unsigned short    c;
  unsigned short    flags;
} VECTOR4D_INT;

typedef struct vector2D_t
{
  float   u;
  float   v;
} VECTOR2D;

typedef struct material_t
{
  char      *name;
  char      *texture;
  SDLGL_TEX id_tex;
  //int       id_tex;
  COLOR     color;
  /* Other material parameters follow */
} MATERIAL;

typedef struct mesh3D_t
{
  long         n_vertex;
  long         n_poly;
  long         n_map;
  VECTOR3D     *vertex;
  VECTOR3D     *normal;
  VECTOR4D_INT *poly;
  VECTOR2D     *map;
  int          mat;
} MESH3D;

typedef struct object3D_t
{
  char      *name;
  int       indx;       /* Current mesh being processed */
  int       mat_indx;   /* Current material being processed */
  int       n_meshes;
  int       n_mats;
  MESH3D    *mesh;
  MATERIAL  *mat_list;
} OBJ3D;

/* Prototypes */
int loader_3ds (OBJ3D *obj, char *fname);
#endif
