/*
 * Priscila: A GNU/EDMA OpenGL Presentation Tool
 * Copyright (C) 2004  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
 * GL_ITEM_IMAGE Class
 * Encapsulates a image presentation item object
 * -------------------------------------------------------------
 * September, 11th, 2004
 * - File Creation. GNU/EDMA in-line class
 * -------------------------------------------------------------
 * September, 12th, 2004
 * - Added GL_SLOW_ZOOM and GL_FLYING_QUADS effects
 */

/* OpenGL includes */
#include <GL/gl.h>
#include <GL/glu.h>

#include <edma.h>
#include <stdlib.h>

#include <time.h>

typedef struct
{
  ESint32      indx;
  EReal64      step;
  EReal64      min, max;
} GL_FX_SLOW_ZOOMData;

typedef struct
{
  ESint32      indx;
  EReal64      stepx, stepy, stepz;
} GL_SPIN_OBJECTData;

typedef struct
{
  ESint32      indx;
  ESint32      frames;
  ESint32      skip_frames;
  ESint32      current_frame;
} GL_ANIM_OBJECTData;

typedef struct
{
  EReal64   vel;
  EReal64   sizex, sizey;
  EReal64   x, y, z;
  EReal64   alpha;
} PARTICLE;

typedef struct
{
  ESint32      indx;
  ESint32      n;
  EDMAT_BUFFER quad;
} GL_FLYING_QUADSData;

ESint32 EDMAPROC GL_FX_SLOW_ZOOMborn (OBJID id);
ESint32 EDMAPROC GL_FX_SLOW_ZOOMrip (OBJID id);

ESint32 EDMAPROC GL_FX_SLOW_ZOOMrun (OBJID id, OBJID *item);
ESint32 EDMAPROC GL_FX_SLOW_ZOOMload (OBJID id, OBJID s);
ESint32 EDMAPROC GL_FX_SLOW_ZOOMsave (OBJID id, OBJID s);

ESint32 EDMAPROC GL_SPIN_OBJECTborn (OBJID id);
ESint32 EDMAPROC GL_SPIN_OBJECTrip (OBJID id);

ESint32 EDMAPROC GL_SPIN_OBJECTrun (OBJID id, OBJID *item);
ESint32 EDMAPROC GL_SPIN_OBJECTload (OBJID id, OBJID s);
ESint32 EDMAPROC GL_SPIN_OBJECTsave (OBJID id, OBJID s);

ESint32 EDMAPROC GL_ANIM_OBJECTborn (OBJID id);
ESint32 EDMAPROC GL_ANIM_OBJECTrip (OBJID id);

ESint32 EDMAPROC GL_ANIM_OBJECTrun (OBJID id, OBJID *item);
ESint32 EDMAPROC GL_ANIM_OBJECTload (OBJID id, OBJID s);
ESint32 EDMAPROC GL_ANIM_OBJECTsave (OBJID id, OBJID s);



ESint32 EDMAPROC GL_FLYING_QUADSborn (OBJID id);
ESint32 EDMAPROC GL_FLYING_QUADSrip (OBJID id);

ESint32 EDMAPROC GL_FLYING_QUADSrun (OBJID id, OBJID *item);
ESint32 EDMAPROC GL_FLYING_QUADSload (OBJID id, OBJID s);
ESint32 EDMAPROC GL_FLYING_QUADSsave (OBJID id, OBJID s);

ESint32
gl_fx_slow_zoom_class_factory ()
{
  CLASSID    cid;

  cid = edma_idf_get_free_class_id (EDMA_LOCAL_CLASS);

  edma_idf_set_class_name (cid, "GL_FX_SLOW_ZOOM");
  edma_idf_set_class_namespace (cid, "PRISCILA");
  edma_idf_set_class_version (cid, 0, 0);

  /* Item position in 3D space*/
  edma_add_local_class_property (cid, "indx", DT_EOBJECT, E_L, 0);
  edma_add_local_class_property (cid, "step", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "min", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "max", DT_EREAL64, E_L, 0);

  /* GL_ITEM methods */
  edma_add_local_class_method (cid, "born", "", (PPROC) GL_FX_SLOW_ZOOMrip, 0, 0, 0);
  edma_add_local_class_method (cid, "rip", "", (PPROC) GL_FX_SLOW_ZOOMrip, 0, 0, 0);

  edma_add_local_class_method (cid, "run", "", (PPROC) GL_FX_SLOW_ZOOMrun, 0, 0, 0);
  edma_add_local_class_method (cid, "save", "O", (PPROC) GL_FX_SLOW_ZOOMsave, 0, 0, 0);
  edma_add_local_class_method (cid, "load", "O", (PPROC) GL_FX_SLOW_ZOOMload, 0, 0, 0);

  edma_idf_set_class_id (cid);
  return 0;
}


ESint32 EDMAPROC 
GL_FX_SLOW_ZOOMborn (OBJID obj)
{
  GL_FX_SLOW_ZOOMData   *m = (GL_FX_SLOW_ZOOMData*) edma_get_data_ref (obj);

  /* Set default values */
  //m->obj = -1;
  m->min = -20;
  m->max = -30;
  m->step = 0.01f;

  return 0;
}
ESint32 EDMAPROC 
GL_FX_SLOW_ZOOMrip (OBJID obj)
{
  GL_FX_SLOW_ZOOMData   *m = (GL_FX_SLOW_ZOOMData*) edma_get_data_ref (obj);

  /* Does Nothing at this moment*/

  return 0;
}

ESint32 EDMAPROC 
GL_FX_SLOW_ZOOMrun (OBJID obj, OBJID *item)
{
  EReal64               x, y, z;
  GL_FX_SLOW_ZOOMData   *m = (GL_FX_SLOW_ZOOMData*) edma_get_data_ref (obj);


  edma_met3 (item[m->indx], "get_pos", &x, &y, &z);
  //edma_printf_obj (obj, "Updating object '%d' step %lf pos %lf min %lf", m->obj, m->step, z, m->min);
  
  /* Update z pos */
  if (z > m->min) 
    z = m->max;
  else
    z += m->step;  

  edma_met3 (item[m->indx], "set_pos", x, y, z);

  return 0;
}

ESint32 EDMAPROC 
GL_FX_SLOW_ZOOMload (OBJID obj, OBJID s)
{
  EChar                 buffer[1024];
  ESint32               len;
  GL_FX_SLOW_ZOOMData   *m = (GL_FX_SLOW_ZOOMData*) edma_get_data_ref (obj);
  
  len = 1024;
  edma_met3 (s, "gets", buffer, &len);
  buffer[strlen(buffer) - 1] = 0; /* Chomp */

  sscanf (buffer, "%d %lf %lf %lf",(int*) &m->indx, &m->min, &m->max, &m->step);

  edma_printf_obj (obj, "INDX: %d | MIN: %lf | MAX: %lf | STEP: %lf", m->indx, m->min, m->max, m->step);

  return obj;
}

ESint32 EDMAPROC 
GL_FX_SLOW_ZOOMsave (OBJID obj, OBJID s)
{
  GL_FX_SLOW_ZOOMData   *m = (GL_FX_SLOW_ZOOMData*) edma_get_data_ref (obj);

  edma_printf_obj (obj, "Method 'load' still not implemented");

  return 0;
}

/************************************************************************************/


ESint32
gl_spin_object_class_factory ()
{
  CLASSID    cid;

  cid = edma_idf_get_free_class_id (EDMA_LOCAL_CLASS);

  edma_idf_set_class_name (cid, "GL_SPIN_OBJECT");
  edma_idf_set_class_namespace (cid, "PRISCILA");
  edma_idf_set_class_version (cid, 0, 0);

  /* Item position in 3D space*/
  edma_add_local_class_property (cid, "indx", DT_EOBJECT, E_L, 0);
  edma_add_local_class_property (cid, "stepx", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "stepy", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "stepz", DT_EREAL64, E_L, 0);

  /* GL_ITEM methods */
  edma_add_local_class_method (cid, "born", "", (PPROC) GL_SPIN_OBJECTrip, 0, 0, 0);
  edma_add_local_class_method (cid, "rip", "", (PPROC) GL_SPIN_OBJECTrip, 0, 0, 0);

  edma_add_local_class_method (cid, "run", "", (PPROC) GL_SPIN_OBJECTrun, 0, 0, 0);
  edma_add_local_class_method (cid, "save", "O", (PPROC) GL_SPIN_OBJECTsave, 0, 0, 0);
  edma_add_local_class_method (cid, "load", "O", (PPROC) GL_SPIN_OBJECTload, 0, 0, 0);

  edma_idf_set_class_id (cid);
  return 0;
}


ESint32 EDMAPROC 
GL_SPIN_OBJECTborn (OBJID obj)
{
  GL_SPIN_OBJECTData   *m = (GL_SPIN_OBJECTData*) edma_get_data_ref (obj);

  /* Set default values */
  m->stepx = m->stepy = m->stepz = 0.0f;

  return 0;
}
ESint32 EDMAPROC 
GL_SPIN_OBJECTrip (OBJID obj)
{
  GL_SPIN_OBJECTData   *m = (GL_SPIN_OBJECTData*) edma_get_data_ref (obj);

  /* Does Nothing at this moment*/

  return 0;
}

ESint32 EDMAPROC 
GL_SPIN_OBJECTrun (OBJID obj, OBJID *item)
{
  EReal64               x, y, z;
  GL_SPIN_OBJECTData   *m = (GL_SPIN_OBJECTData*) edma_get_data_ref (obj);


  edma_met3 (item[m->indx], "get_rot", &x, &y, &z);
  //edma_printf_obj (obj, "Updating object '%d' step %lf pos %lf min %lf", m->obj, m->step, z, m->min);
  
  /* Update z pos */
  x += m->stepx;  
  y += m->stepy;  
  z += m->stepz;  

  edma_met3 (item[m->indx], "set_rot", x, y, z);

  return 0;
}

ESint32 EDMAPROC 
GL_SPIN_OBJECTload (OBJID obj, OBJID s)
{
  EChar                 buffer[1024];
  ESint32               len;
  GL_SPIN_OBJECTData   *m = (GL_SPIN_OBJECTData*) edma_get_data_ref (obj);
  
  len = 1024;
  edma_met3 (s, "gets", buffer, &len);
  buffer[strlen(buffer) - 1] = 0; /* Chomp */

  sscanf (buffer, "%d %lf %lf %lf",(int*) &m->indx, &m->stepx, &m->stepy, &m->stepz);

  edma_printf_obj (obj, "INDX: %d | MIN: %lf | MAX: %lf | STEP: %lf", m->indx, m->stepx, m->stepy, m->stepz);

  return obj;
}

ESint32 EDMAPROC 
GL_SPIN_OBJECTsave (OBJID obj, OBJID s)
{
  GL_SPIN_OBJECTData   *m = (GL_SPIN_OBJECTData*) edma_get_data_ref (obj);

  edma_printf_obj (obj, "Method 'load' still not implemented");

  return 0;
}

/************************************************************************************/

/************************************************************************************/
/*************************************************************************************/
ESint32
gl_anim_object_class_factory ()
{
  CLASSID    cid;

  cid = edma_idf_get_free_class_id (EDMA_LOCAL_CLASS);

  edma_idf_set_class_name (cid, "GL_ANIM_OBJECT");
  edma_idf_set_class_namespace (cid, "PRISCILA");
  edma_idf_set_class_version (cid, 0, 0);

  /* ANIM_OBJECT Properties */
  edma_add_local_class_property (cid, "indx", DT_EOBJECT, E_L, 0);
  edma_add_local_class_property (cid, "frames", DT_EUINT32, E_L, 0);
  edma_add_local_class_property (cid, "skip_frames", DT_EUINT32, E_L, 0);
  edma_add_local_class_property (cid, "current_frame", DT_EUINT32, E_L, 0);

  /* GL_ANIM_OBJECT methods */
  edma_add_local_class_method (cid, "born", "", (PPROC) GL_ANIM_OBJECTrip, 0, 0, 0);
  edma_add_local_class_method (cid, "rip", "",  (PPROC) GL_ANIM_OBJECTrip, 0, 0, 0);

  edma_add_local_class_method (cid, "run", "",   (PPROC) GL_ANIM_OBJECTrun, 0, 0, 0);
  edma_add_local_class_method (cid, "save", "O", (PPROC) GL_ANIM_OBJECTsave, 0, 0, 0);
  edma_add_local_class_method (cid, "load", "O", (PPROC) GL_ANIM_OBJECTload, 0, 0, 0);

  /* Animation points are defined in superclasses -> Aggregation like form */
  edma_add_local_class_superclass_by_name (cid, "GL_ITEM", "INC", "ANIM");
  edma_add_local_class_superclass_by_name (cid, "GL_ITEM", "INITIAL", "ANIM");
  edma_add_local_class_superclass_by_name (cid, "GL_ITEM", "END", "ANIM");


  edma_idf_set_class_id (cid);
  return 0;
}


ESint32 EDMAPROC 
GL_ANIM_OBJECTborn (OBJID obj)
{
  GL_ANIM_OBJECTData   *m = (GL_ANIM_OBJECTData*) edma_get_data_ref (obj);

  /* Does nothing at this moment */
  m->frames = m->skip_frames = m->current_frame = 0;
  return 0;
}
ESint32 EDMAPROC 
GL_ANIM_OBJECTrip (OBJID obj)
{
  GL_ANIM_OBJECTData   *m = (GL_ANIM_OBJECTData*) edma_get_data_ref (obj);

  /* Does Nothing at this moment*/

  return 0;
}

ESint32 EDMAPROC 
GL_ANIM_OBJECTrun (OBJID obj, OBJID *item)
{
  EReal64               ix, iy, iz, ia;
  EReal64               x, y, z, a;
  EReal64               x1, y1, z1, a1;
  GL_ANIM_OBJECTData   *m = (GL_ANIM_OBJECTData*) edma_get_data_ref (obj);

  //edma_printf_obj (obj, "Current Frame is: %ld", m->current_frame);
  m->current_frame++;
  if (m->current_frame < m->skip_frames)
    return 0;
  /* Update Pos */
  edma_met3 (obj, "INC>get_pos", &ix, &iy, &iz);
  if ((ix != 0) || (iy != 0) || (iz != 0))
    {
      edma_met3 (item[m->indx], "get_pos", &x, &y, &z);

      x += ix;  
      y += iy;  
      z += iz;  

      edma_met3 (item[m->indx], "set_pos", x, y, z);
      /* Check end condition */
      edma_met3 (obj, "END>get_pos", &x1, &y1, &z1);
      if (fabs(x - x1) < 0.01) ix = 0;
      if (fabs(y - y1) < 0.01) iy = 0;
      if (fabs(z - z1) < 0.01) iz = 0;
      edma_met3 (obj, "INC>set_pos", ix, iy, iz);
    }

  /* Update Angle */
  edma_met3 (obj, "INC>get_scale", &ix, &iy, &iz);
  if ((ix != 0) || (iy != 0) || (iz != 0))
    {
      edma_met3 (item[m->indx], "get_scale", &x, &y, &z);

      x += ix;  
      y += iy;  
      z += iz;  

      edma_met3 (item[m->indx], "set_scale", x, y, z);
      /* Check end condition */
      edma_met3 (obj, "END>get_scale", &x1, &y1, &z1);
      if (fabs(x - x1) < 0.01) ix = 0;
      if (fabs(y - y1) < 0.01) iy = 0;
      if (fabs(z - z1) < 0.01) iz = 0;

      edma_met3 (obj, "INC>set_scale", ix, iy, iz);
    }


  /* Update Angle */
  edma_met3 (obj, "INC>get_rot", &ix, &iy, &iz);
  if ((ix != 0) || (iy != 0) || (iz != 0))
    {
      edma_met3 (item[m->indx], "get_rot", &x, &y, &z);

      x += ix;  
      y += iy;  
      z += iz;  

      edma_met3 (item[m->indx], "set_rot", x, y, z);
      /* Check end condition */
      edma_met3 (obj, "END>get_rot", &x1, &y1, &z1);
      if (fabs(x - x1) < 0.01) ix = 0;
      if (fabs(y - y1) < 0.01) iy = 0;
      if (fabs(z - z1) < 0.01) iz = 0;
      edma_met3 (obj, "INC>set_rot", ix, iy, iz);
    }

  /* Update Color + Transparency  */
  edma_met3 (obj, "INC>get_color", &ix, &iy, &iz, &ia);

  if ((ix != 0) || (iy != 0) || (iz != 0) || (ia != 0))
    {
      edma_met3 (item[m->indx], "get_color", &x, &y, &z, &a);

      x += ix;  
      y += iy;  
      z += iz;  
      a += ia;

      edma_met3 (item[m->indx], "set_color", x, y, z, a);
      /* Check end condition */
      edma_met3 (obj, "END>get_color", &x1, &y1, &z1, &a1);
      if (fabs(x - x1) < 0.01) ix = 0.0;
      if (fabs(y - y1) < 0.01) iy = 0.0;
      if (fabs(z - z1) < 0.01) iz = 0.0;
      if (fabs(a - a1) < 0.01) ia = 0.0;
	
      edma_met3 (obj, "INC>set_color", ix, iy, iz, ia);
    }

  return 0;
}

ESint32 EDMAPROC 
GL_ANIM_OBJECTload (OBJID obj, OBJID s)
{
  EChar                 buffer[1024];
  ESint32               len;
  double                x, y, z, a;
  double                x1, y1, z1, a1;
  double                x2, y2, z2, a2;
  GL_ANIM_OBJECTData   *m = (GL_ANIM_OBJECTData*) edma_get_data_ref (obj);
  
  len = 1024;
  edma_met3 (s, "gets", buffer, &len);
  buffer[strlen(buffer) - 1] = 0; /* Chomp */

  sscanf (buffer, "%d %ld %ld",(int*) &m->indx, &m->skip_frames, &m->frames);

  edma_printf_obj (obj, "INDX: %d | Frames: %ld", m->indx, m->frames);

  /* Load animation data*/
  edma_printf_obj (obj, "%s", "Loading Initial Item conditions");
  edma_met3 (obj, "INITIAL>.load", s);
  edma_printf_obj (obj, "%s", "Loading Final Item conditions");
  edma_met3 (obj, "END>.load", s);

  /* Calculate INCrement */
  edma_printf_obj (obj, "%s", "Calculating animation increments");
  edma_met3 (obj, "INITIAL>get_pos", &x1, &y1, &z1);
  edma_met3 (obj, "END>get_pos", &x2, &y2, &z2);
  edma_met3 (obj, "INC>set_pos", 
	     (x2 - x1) / m->frames,
	     (y2 - y1) / m->frames,
	     (z2 - z1) / m->frames);

  edma_printf ("Position Increment: %lf, %lf, %lf",
	       (x2 - x1) / m->frames,
	       (y2 - y1) / m->frames,
	       (z2 - z1) / m->frames);	       
  
  edma_met3 (obj, "INITIAL>get_rot", &x1, &y1, &z1);
  edma_met3 (obj, "END>get_rot", &x2, &y2, &z2);
  edma_met3 (obj, "INC>set_rot", 
	     (x2 - x1) / m->frames,
	     (y2 - y1) / m->frames,
	     (z2 - z1) / m->frames);

  edma_met3 (obj, "INITIAL>get_scale", &x1, &y1, &z1);
  edma_met3 (obj, "END>get_scale", &x2, &y2, &z2);
  edma_met3 (obj, "INC>set_scale", 
	     (x2 - x1) / m->frames,
	     (y2 - y1) / m->frames,
	     (z2 - z1) / m->frames);

  edma_met3 (obj, "INITIAL>get_color", &x1, &y1, &z1, &a1);
  edma_met3 (obj, "END>get_color", &x2, &y2, &z2, &a2);
  edma_met3 (obj, "INC>set_color", 
	     (x2 - x1) / m->frames,
	     (y2 - y1) / m->frames,
	     (z2 - z1) / m->frames,
	     (a2 - a1) / m->frames);



  return obj;
}

ESint32 EDMAPROC 
GL_ANIM_OBJECTsave (OBJID obj, OBJID s)
{
  GL_ANIM_OBJECTData   *m = (GL_ANIM_OBJECTData*) edma_get_data_ref (obj);

  edma_printf_obj (obj, "Method 'load' still not implemented");

  return 0;
}

/************************************************************************************/
/************************************************************************************/




ESint32
gl_flying_quads_class_factory ()
{
  CLASSID    cid;

  cid = edma_idf_get_free_class_id (EDMA_LOCAL_CLASS);

  edma_idf_set_class_name (cid, "GL_FLYING_QUADS");
  edma_idf_set_class_namespace (cid, "PRISCILA");
  edma_idf_set_class_version (cid, 0, 0);

  /* Item position in 3D space*/
  edma_add_local_class_property (cid, "indx", DT_EOBJECT, E_L, 0);
  edma_add_local_class_property (cid, "num", DT_ESINT32, E_L, 0);
  edma_add_local_class_property (cid, "data", DT_EBUFFER, E_L, 0);

  /* GL_ITEM methods */
  edma_add_local_class_method (cid, "born", "", (PPROC) GL_FLYING_QUADSrip, 0, 0, 0);
  edma_add_local_class_method (cid, "rip", "", (PPROC) GL_FLYING_QUADSrip, 0, 0, 0);

  edma_add_local_class_method (cid, "run", "", (PPROC) GL_FLYING_QUADSrun, 0, 0, 0);
  edma_add_local_class_method (cid, "save", "O", (PPROC) GL_FLYING_QUADSsave, 0, 0, 0);
  edma_add_local_class_method (cid, "load", "O", (PPROC) GL_FLYING_QUADSload, 0, 0, 0);

  edma_idf_set_class_id (cid);
  return 0;
}


ESint32 EDMAPROC 
GL_FLYING_QUADSborn (OBJID obj)
{
  GL_FLYING_QUADSData   *m = (GL_FLYING_QUADSData*) edma_get_data_ref (obj);

  srand (time(NULL));

  return 0;
}
ESint32 EDMAPROC 
GL_FLYING_QUADSrip (OBJID obj)
{
  GL_FLYING_QUADSData   *m = (GL_FLYING_QUADSData*) edma_get_data_ref (obj);

  edma_buffer_free (&m->quad);

  return 0;
}

ESint32 EDMAPROC 
GL_FLYING_QUADSrun (OBJID obj, OBJID *item)
{
  EReal64               x, y, z;
  ESint32               i;
  PARTICLE              *p;
  GL_FLYING_QUADSData   *m = (GL_FLYING_QUADSData*) edma_get_data_ref (obj);

  p = (PARTICLE *) m->quad.dat;

  glBlendFunc (GL_SRC_ALPHA, GL_ONE);
  glEnable (GL_BLEND);
  glDisable (GL_DEPTH_TEST);
  glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);
  
  for (i = 0; i < m->n; i++)
    {
      /* Update Particle */
      p[i].x = p[i].x - p[i].vel;  /* Move right to left */
      if (p[i].x < -15.0) p[i].x = 15.0;

      /* Transform Particle */
      glLoadIdentity();

      glTranslatef (p[i].x, p[i].y , p[i].z);
      glScalef (p[i].sizex, p[i].sizey, p[i].sizex);

      glBindTexture (GL_TEXTURE_2D, 0);
      glBegin(GL_QUADS);
      
      glNormal3f( 0.0f, 0.0f, 0.1f); /* Normal is Up*/
      
      //glColor4f ( 0.2f, 0.2f, 0.8f, 0.99f);
      glColor4f (0.8f, 0.8f, 1.0f, p[i].alpha);
      glVertex3f(  1.0f,  1.0f,  0.0f);
      glVertex3f(  1.0f, -1.0f,  0.0f);
      glVertex3f( -1.0f, -1.0f,  0.0f);
      glVertex3f( -1.0f, 1.0f,  0.0f);
      
      glEnd();
    }


  return 0;
}

ESint32 EDMAPROC 
GL_FLYING_QUADSload (OBJID obj, OBJID s)
{
  EChar                 buffer[1024];
  EReal64               velocity, z, size, y_base, y_delta, alpha;
  PARTICLE              *part;
  ESint32               len, i;
  GL_FLYING_QUADSData   *m = (GL_FLYING_QUADSData*) edma_get_data_ref (obj);
  
  len = 1024;
  edma_met3 (s, "gets", buffer, &len);
  buffer[strlen(buffer) - 1] = 0; /* Chomp */

  sscanf (buffer, "%d %lf %lf %lf %lf %lf %lf", 
	  (int*)&m->n, &velocity, &size, &z, &y_base, &y_delta, &alpha);

  edma_printf_obj (obj, "NUM: %d | VEL: %lf | SIZE: %lf | Z: %lf Y0:%lf dY:%lf Alpha:%ld", 
		   m->n, velocity, size, z, y_base, y_delta, alpha);

  /* Alloca buffer */
  if ((edma_buffer_alloc (&m->quad, sizeof(PARTICLE) * m->n)) == -1)
    {
      edma_printf_err ("%s", "Cannot allocate memory for particles");
      return -1;
    }

  part = (PARTICLE *) m->quad.dat;

  for (i = 0; i < m->n; i++)
    {
      part[i].vel = velocity * ((double)rand()/ RAND_MAX);
      part[i].z   = z;
      part[i].x   = 10.0 * (double)rand () / RAND_MAX;
      part[i].y   = y_base + y_delta  * (((double)rand () / RAND_MAX) - 0.5f);
      part[i].sizex =  size * ((double)rand () / (double)RAND_MAX);
      part[i].sizey = size * ((double)rand () / (double)RAND_MAX);
      part[i].alpha = alpha * (double)rand() /RAND_MAX;

      if (part[i].sizex < 0.5) part[i].sizex = 0.5;
      if (part[i].sizey < 0.5) part[i].sizey = 0.5;
    }

  /* Particle system initiated */

  return obj;
}

ESint32 EDMAPROC 
GL_FLYING_QUADSsave (OBJID obj, OBJID s)
{
  GL_FLYING_QUADSData   *m = (GL_FLYING_QUADSData*) edma_get_data_ref (obj);

  edma_printf_obj (obj, "Method 'load' still not implemented");

  return 0;
}
