/*
 * Priscila: A GNU/EDMA OpenGL Presentation Tool
 * Copyright (C) 2004  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
 * GL_ITEM Class
 * Encapsulates an abstract presentation object 
 * -------------------------------------------------------------
 * September, 4th, 2004
 * - File Creation. GNU/EDMA in-line class
 * -------------------------------------------------------------
 * September, 8th, 2004
 * - Adding save/load methods 
 * --------------------------------------------------------------
 * September, 9th, 2004
 * - Finished load method
 * - Added vector accessor methods
 */

#include <stdio.h>
#include <strings.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include <edma.h>
#include "gl_item.h"


/* Class data structure for property mapping*/
typedef struct gl_item_t
{
  double     xpos, ypos, zpos;
  double     xrot, yrot, zrot;
  double     xscale, yscale, zscale;
  double     r, g, b, alpha;
} GL_ITEM_Data;

/* Local prototypes  */
ESint32 EDMAPROC GL_ITEMset_pos   (OBJID, EReal64, EReal64, EReal64);
ESint32 EDMAPROC GL_ITEMset_rot   (OBJID, EReal64, EReal64, EReal64);
ESint32 EDMAPROC GL_ITEMset_scale (OBJID, EReal64, EReal64, EReal64);
ESint32 EDMAPROC GL_ITEMset_color (OBJID, EReal64, EReal64, EReal64, EReal64);

ESint32 EDMAPROC GL_ITEMget_pos   (OBJID, EReal64*, EReal64*, EReal64*);
ESint32 EDMAPROC GL_ITEMget_rot   (OBJID, EReal64*, EReal64*, EReal64*);
ESint32 EDMAPROC GL_ITEMget_scale (OBJID, EReal64*, EReal64*, EReal64*);
ESint32 EDMAPROC GL_ITEMget_color (OBJID, EReal64*, EReal64*, EReal64*, EReal64*);

ESint32 EDMAPROC GL_ITEMrender    (OBJID);
ESint32 EDMAPROC GL_ITEMsave      (OBJID, OBJID);
ESint32 EDMAPROC GL_ITEMload      (OBJID, OBJID);


int
gl_item_class_factory ()
{
  CLASSID    cid;

  cid = edma_idf_get_free_class_id (EDMA_LOCAL_CLASS);

  edma_idf_set_class_name (cid, "GL_ITEM");
  edma_idf_set_class_namespace (cid, "PRISCILA");
  edma_idf_set_class_version (cid, 0, 0);

  /* Item position in 3D space*/
  edma_add_local_class_property (cid, "xpos", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "ypos", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "zpos", DT_EREAL64, E_L, 0);

  /* Item rotation angles in 3D space*/
  edma_add_local_class_property (cid, "xrot", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "yrot", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "zrot", DT_EREAL64, E_L, 0);

  /* Item scaling factors in 3D space*/
  edma_add_local_class_property (cid, "xscale", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "yscale", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "zscale", DT_EREAL64, E_L, 0);  

  /* Item color information */
  edma_add_local_class_property (cid, "r", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "g", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "b", DT_EREAL64, E_L, 0);
  edma_add_local_class_property (cid, "alpha", DT_EREAL64, E_L, 0);


  /* GL_ITEM methods */
  edma_add_local_class_method (cid, "set_pos", "R32R32R32", (PPROC) GL_ITEMset_pos, 0, 0, 0);
  edma_add_local_class_method (cid, "set_rot", "R32R32R32", (PPROC) GL_ITEMset_rot, 0, 0, 0);
  edma_add_local_class_method (cid, "set_scale", "R32R32R32", (PPROC) GL_ITEMset_scale, 0, 0, 0);
  edma_add_local_class_method (cid, "set_color", "R32R32R32R32", (PPROC) GL_ITEMset_color, 0, 0, 0);

  edma_add_local_class_method (cid, "get_pos", "sR32sR32sR32", (PPROC) GL_ITEMget_pos, 0, 0, 0);
  edma_add_local_class_method (cid, "get_rot", "sR32sR32sR32", (PPROC) GL_ITEMget_rot, 0, 0, 0);
  edma_add_local_class_method (cid, "get_scale", "sR32sR32sR32", (PPROC) GL_ITEMget_scale, 0, 0, 0);
  edma_add_local_class_method (cid, "get_color", "sR32sR32sR32sR32", (PPROC) GL_ITEMget_color, 0, 0, 0);


  edma_add_local_class_method (cid, "render", "", (PPROC) GL_ITEMrender, 0, 0, 0);
  edma_add_local_class_method (cid, "save", "O", (PPROC) GL_ITEMsave, 0, 0, 0);
  edma_add_local_class_method (cid, "load", "O", (PPROC) GL_ITEMload, 0, 0, 0);

  //edma_local_class_finish (cid);
  edma_idf_set_class_id (cid);


  return 0;
}


/* Method Implementation for GL_ITEM Methods */
ESint32 EDMAPROC
GL_ITEMset_pos (OBJID obj, EReal64 xpos, EReal64 ypos, EReal64 zpos)
{
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  m->xpos = xpos;
  m->ypos = ypos;
  m->zpos = zpos;

  return 0;
}

ESint32 EDMAPROC
GL_ITEMset_rot (OBJID obj, EReal64 xrot, EReal64 yrot, EReal64 zrot)
{
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  m->xrot = xrot;
  m->yrot = yrot;
  m->zrot = zrot;

  return 0;
}

ESint32 EDMAPROC
GL_ITEMset_scale (OBJID obj, EReal64 xscale, EReal64 yscale, EReal64 zscale)
{
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  m->xscale = xscale;
  m->yscale = yscale;
  m->zscale = zscale;
  

  return 0;
}

ESint32 EDMAPROC
GL_ITEMset_color (OBJID obj, EReal64 r, EReal64 g, EReal64 b, EReal64 alpha)
{
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  m->r = r;
  m->g = g;
  m->b = b;
  m->alpha = alpha;

  return 0;
}


ESint32 EDMAPROC
GL_ITEMget_pos (OBJID obj, EReal64 *xpos, EReal64 *ypos, EReal64 *zpos)
{
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  *xpos = m->xpos;
  *ypos = m->ypos;
  *zpos = m->zpos;

  return 0;
}

ESint32 EDMAPROC
GL_ITEMget_rot (OBJID obj, EReal64 *xrot, EReal64 *yrot, EReal64 *zrot)
{
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  *xrot = m->xrot;
  *yrot = m->yrot;
  *zrot = m->zrot;

  return 0;
}

ESint32 EDMAPROC
GL_ITEMget_scale (OBJID obj, EReal64 *xscale, EReal64 *yscale, EReal64 *zscale)
{
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  *xscale = m->xscale;
  *yscale = m->yscale;
  *zscale = m->zscale;
  

  return 0;
}

ESint32 EDMAPROC
GL_ITEMget_color (OBJID obj, EReal64 *r, EReal64 *g, EReal64 *b, EReal64 *alpha)
{
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  *r = m->r;
  *g = m->g;
  *b = m->b;
  *alpha = m->alpha;

  return 0;
}

/*********************************************************************************/
ESint32 EDMAPROC
GL_ITEMrender (OBJID obj)
{
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

#if 1
  glEnable  (GL_BLEND);
  glDisable (GL_DEPTH_TEST);
  
  glBlendFunc (GL_SRC_ALPHA, GL_ONE);
  //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	
#endif 
  /* Set Color */
  glColor4f (m->r, m->g, m->b, m->alpha);

  /* Set Geometry */
  glLoadIdentity ();

  /* Run common transformation for all items */
#if 0
  edma_printf_obj (obj, "Position : %lf, %lf, %lf", m->xpos, m->ypos, m->zpos);
  edma_printf_obj (obj, "Scale: %lf, %lf, %lf", m->xscale, m->yscale, m->zscale);
  edma_printf_obj (obj, "Rotation: %lf, %lf, %lf", m->xrot, m->yrot, m->zrot);
  edma_printf_obj (obj, "Color: %lf, %lf, %lf %lf", m->r, m->g, m->b, m->alpha);
#endif 

  glTranslatef (m->xpos, m->ypos, m->zpos);

  glScalef     (m->xscale, m->yscale, m->zscale);

  glRotatef    (m->xrot, 1.0f, 0.0f, 0.0f);
  glRotatef    (m->yrot, 0.0f, 1.0f, 0.0f);
  glRotatef    (m->zrot, 0.0f, 0.0f, 1.0f);

  /* Real render of object is implemented on subclasses */
#if 0
  if (m->alpha == 1.0)
    {
      glDisable  (GL_BLEND);
      glEnable (GL_DEPTH_TEST);
    }
  else
    {
      glEnable (GL_BLEND);
      glEnable(GL_DEPTH_TEST);
    }
#endif
  return 0;
}

ESint32 EDMAPROC
GL_ITEMsave (OBJID obj, OBJID stream)
{
  EChar          buffer[1024];
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  /* Dumping data to stream */
  snprintf (buffer, 1024, "%f %f %f\n", m->xpos, m->ypos, m->zpos);
  edma_met3 (stream, "puts", buffer);

  snprintf (buffer, 1024, "%f %f %f\n", m->xrot, m->yrot, m->zrot);
  edma_met3 (stream, "puts", buffer);

  snprintf (buffer, 1024, "%f %f %f\n", m->xscale, m->yscale, m->zscale);
  edma_met3 (stream, "puts", buffer);

  snprintf (buffer, 1024, "%f %f %f %f\n", m->r, m->g, m->b, m->alpha);
  edma_met3 (stream, "puts", buffer);

  return 0;
}

ESint32 EDMAPROC
GL_ITEMload (OBJID obj, OBJID stream)
{
  EChar            buffer[1024];
  ESint32          len;
  GL_ITEM_Data   *m = edma_get_data_ref (obj);

  /* Read Subclass info */
  len = 1024;
  edma_met3 (stream, "gets", buffer, &len);
  buffer[strlen(buffer) - 1] = 0; /* chomp buffer */
  edma_printf_obj (obj, "'%s' object data follows", buffer);
  if (strlen (buffer) == 0)
    return -1;

  /* Dynamic specialization of GL_ITEM object */
  edma_add_subclass (obj, edma_get_class_id (buffer), 
		     "INNER", "SUPER");

  /* Read common GL_ITEM data from file */
  len = 1024;
  edma_met3 (stream, "gets", buffer, &len);
  sscanf (buffer, "%lf %lf %lf", &m->xpos, &m->ypos, &m->zpos);


  len = 1024;
  edma_met3 (stream, "gets", buffer, &len);
  sscanf (buffer, "%lf %lf %lf", &m->xrot, &m->yrot, &m->zrot);

  /* Read Scale */
  len = 1024;
  edma_met3 (stream, "gets", buffer, &len);
  sscanf (buffer, "%lf %lf %lf", &m->xscale, &m->yscale, &m->zscale);

  /* Read Color */
  len = 1024;
  edma_met3 (stream, "gets", buffer, &len);
  sscanf (buffer, "%lf %lf %lf %lf", &m->r, &m->g, &m->b, &m->alpha);

  edma_met3 (obj, "INNER<.load", stream);

  /*return edma_downcast_obj (obj, "INNER");*/
  return obj;
}



