/*
 * Priscila: A GNU/EDMA OpenGL Presentation Tool
 * Copyright (C) 2004  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
 * GL_ITEM_3DS Class
 * Encapsulates a image presentation item object
 * -------------------------------------------------------------
 * September, 14th, 2004
 * - File Creation. GNU/EDMA in-line class
 */

#include <stdlib.h>

/* OpenGL includes */
#include <GL/gl.h>
#include <GL/glu.h>

/* SDL and SDL TrueType Font include */
#include <SDL.h>

#include <edma.h>

#include "3ds.h"
#include "sdlgl_tex.h"

/* Data structures */
/* -- Data structure for GL_ITEM_3DS */
typedef struct
{
  EPChar      data;
  EPVoid      handler;
} GL_ITEM_3DSData;


/* -- Method prototypes for GL_ITEM_3DS Class*/

ESint32 EDMAPROC GL_ITEM_3DScreate (OBJID, EPChar);
ESint32 EDMAPROC GL_ITEM_3DSdestroy (OBJID);
ESint32 EDMAPROC GL_ITEM_3DSrip (OBJID);
ESint32 EDMAPROC GL_ITEM_3DSrender (OBJID);
ESint32 EDMAPROC GL_ITEM_3DSsave (OBJID, OBJID);
ESint32 EDMAPROC GL_ITEM_3DSload (OBJID, OBJID);


ESint32
gl_item_3ds_class_factory ()
{
  CLASSID    cid;

  cid = edma_idf_get_free_class_id (EDMA_LOCAL_CLASS);

  edma_idf_set_class_name (cid, "GL_ITEM_3DS");
  edma_idf_set_class_namespace (cid, "PRISCILA");
  edma_idf_set_class_version (cid, 0, 0);

  /* Item position in 3D space*/
  edma_add_local_class_property (cid, "data", DT_EZSTRING, E_L, 0);
  edma_add_local_class_property (cid, "handler", DT_EPOINTER, E_L, 0);

  /* GL_ITEM methods */
  edma_add_local_class_method (cid, "create", "Z", (PPROC) GL_ITEM_3DScreate, 0, 0, 0);
  edma_add_local_class_method (cid, "destroy", "", (PPROC) GL_ITEM_3DSdestroy, 0, 0, 0);
  edma_add_local_class_method (cid, "rip", "", (PPROC) GL_ITEM_3DSrip, 0, 0, 0);


  edma_add_local_class_method (cid, "render", "", (PPROC) GL_ITEM_3DSrender, 0, 0, 0);
  edma_add_local_class_method (cid, "save", "O", (PPROC) GL_ITEM_3DSsave, 0, 0, 0);
  edma_add_local_class_method (cid, "load", "O", (PPROC) GL_ITEM_3DSload, 0, 0, 0);

  /* GL_ITEM_TEXT inherits from GL_ITEM */
  edma_add_local_class_superclass_by_name (cid, "GL_ITEM", "SUPER", "INNER");

  //edma_local_class_finish (cid);
  edma_idf_set_class_id (cid);


  return 0;
}


ESint32 EDMAPROC 
GL_ITEM_3DScreate (OBJID obj, EPChar filename)
{
  GL_ITEM_3DSData *m = edma_get_data_ref (obj);

  edma_printf_obj (obj, "%s", "Allocating 3ds object memory");
  //if ((m->handler = (void *)malloc (sizeof (obj_type))) == NULL)
  if ((m->handler = (void *)malloc (sizeof (OBJ3D))) == NULL)
    {
      edma_printf_obj (obj, "%s", "Cannot allocate memory for 3D Object");
      return -1;
    }

  edma_printf_obj (obj, "%s", "Loading 3DS Model");
  //if (Load3DS (m->handler, filename) != 1)
  if (loader_3ds (m->handler, filename) < 0)
    {
      edma_printf_obj (obj, "Cannot open 3DS file '%s'", filename);
      free (m->handler);
      return -1;
    }

  /* FIXME: Remove the line bellow
   *        .3ds format includes information about the textures used
   *        by the model. Load3DS should manage texture creation
   */
  //edma_printf_obj (obj, "%s", "Loading model texture");
  //((obj_type*)m->handler)->id_texture=LoadBitmap("spaceshiptexture.bmp");

  /* Store File Handler*/
  edma_wprop3 (obj, "data", filename);

  return 0;
}




ESint32 EDMAPROC 
GL_ITEM_3DSdestroy (OBJID obj)
{
  GL_ITEM_3DSData *m = edma_get_data_ref (obj);

  free (m->handler);

  return 0;
}


ESint32 EDMAPROC 
GL_ITEM_3DSrip (OBJID obj)
{
  GL_ITEM_3DSData *m = edma_get_data_ref (obj);

  free (m->handler);

  return 0;
}


ESint32 EDMAPROC 
GL_ITEM_3DSrender (OBJID obj)
{
  //int l_index;
  //obj_type        *model;
  GLuint            id_texture;
  OBJ3D             *model;
  MESH3D            *mesh;
  double            r,g,b;
  int              i, j;
  GLfloat         p[4] = {.5,.5,0.5, 0.5};
  GL_ITEM_3DSData *m = edma_get_data_ref (obj);

  edma_met3 (obj, "SUPER>render");

  model = (OBJ3D *) m->handler;

  glShadeModel(GL_SMOOTH);	

  glEnable(GL_DEPTH_TEST);
  glDisable(GL_BLEND);
  glDisable(GL_TEXTURE_2D);
  //glEnable(GL_COLOR_MATERIAL);
  //glEnable(GL_CULL_FACE);
  glEnable(GL_LIGHTING);
  //glEnable(GL_LIGHT1);
  glEnable(GL_LIGHT0);
  glEnable(GL_NORMALIZE);
  //glColor3ub(255, 255, 255);   


  //glBindTexture(GL_TEXTURE_2D, model->id_texture); // We set the active texture 

  for (i = 0; i < model->n_meshes; i++)
    {
      mesh = &model->mesh[i];

      if (model->mat_list[mesh->mat].id_tex == NULL)

	{

	  p[0] = (double) model->mat_list[mesh->mat].color.r / 255.0;
	  p[1] = (double) model->mat_list[mesh->mat].color.g / 255.0;
	  p[2] = (double) model->mat_list[mesh->mat].color.b / 255.0;
	  glBegin(GL_TRIANGLES); 

	  for (j = 0; j < mesh->n_poly; j++)
	    {
	      
	      glNormal3f (mesh->normal[j].x, mesh->normal[j].y, mesh->normal[j].z);

	      glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, p);

	      glVertex3f( mesh->vertex[ mesh->poly[j].a ].x,
			  mesh->vertex[ mesh->poly[j].a ].y,
			  mesh->vertex[ mesh->poly[j].a ].z); 
	      
	      glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, p);
	      glVertex3f( mesh->vertex[ mesh->poly[j].b ].x,
			  mesh->vertex[ mesh->poly[j].b ].y,
			  mesh->vertex[ mesh->poly[j].b ].z);
	      
	      glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, p);
	      glVertex3f( mesh->vertex[ mesh->poly[j].c ].x,
			  mesh->vertex[ mesh->poly[j].c ].y,
			  mesh->vertex[ mesh->poly[j].c ].z);
	    }
	  glEnd();
	}
      else  /* We have a texture */
	{
	  glEnable(GL_TEXTURE_2D);
	  //glDisable(GL_CULL_FACE);
	  glDisable(GL_LIGHTING);
	  glDisable(GL_LIGHT0);

	  id_texture = SDLGL_tex_get_gltex (model->mat_list[mesh->mat].id_tex);
	  glBindTexture(GL_TEXTURE_2D, id_texture); 
	  //glBindTexture (GL_TEXTURE_2D, model->mat_list[mesh->mat].id_tex);

	  glBegin(GL_TRIANGLES); 
	  for (j = 0; j < mesh->n_poly; j++)
	    {
	      
	      glNormal3f (mesh->normal[j].x, mesh->normal[j].y, mesh->normal[j].z);

	      glTexCoord2f( mesh->map[ mesh->poly[j].a ].u,
			    mesh->map[ mesh->poly[j].a ].v);
	      
	      glVertex3f( mesh->vertex[ mesh->poly[j].a ].x,
			  mesh->vertex[ mesh->poly[j].a ].y,
			  mesh->vertex[ mesh->poly[j].a ].z); 
	      
	      glTexCoord2f( mesh->map[ mesh->poly[j].b ].u,
			    mesh->map[ mesh->poly[j].b ].v);
	      
	      glVertex3f( mesh->vertex[ mesh->poly[j].b ].x,
			  mesh->vertex[ mesh->poly[j].b ].y,
			  mesh->vertex[ mesh->poly[j].b ].z);
	      
	      glTexCoord2f( mesh->map[ mesh->poly[j].c ].u,
			    mesh->map[ mesh->poly[j].c ].v);

	      glVertex3f( mesh->vertex[ mesh->poly[j].c ].x,
			  mesh->vertex[ mesh->poly[j].c ].y,
			  mesh->vertex[ mesh->poly[j].c ].z);
	    }
	  glEnd();	  
	}
    }


  glDisable(GL_BLEND);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_TEXTURE_2D);
  glDisable(GL_LIGHTING);

  glDisable(GL_CULL_FACE);
  glDisable(GL_LIGHT0);
  glDisable(GL_NORMALIZE);

  return 0;
}


ESint32 EDMAPROC 
GL_ITEM_3DSsave (OBJID obj, OBJID stream)
{
  GL_ITEM_3DSData *m = edma_get_data_ref (obj);

  edma_met3 (stream, "puts", "GL_ITEM_3DS\n");

  /* Let superclass save common information */
  edma_met3 (obj, "SUPER>save", stream);

  edma_met3 (stream, "puts", m->data);
  edma_met3 (stream, "puts", "\n");

  return 0;
}

/* Load method is "INNER" called from superclass GL_ITEM */
ESint32 EDMAPROC 
GL_ITEM_3DSload (OBJID obj, OBJID stream)
{
  EChar            buffer[1024];
  ESint32          len;
  //GL_ITEM_3DSData *m = edma_get_data_ref (obj);

  len = 1024;
  edma_met3 (stream, "gets", buffer, &len);
  buffer[strlen(buffer) - 1] = 0; /* Chomp String */
  
  edma_wprop3 (obj, "data", buffer);
  edma_met3 (obj, "create", buffer);

  return 0;
}


