/*
 * Priscila: A GNU/EDMA OpenGL Presentation Tool
 * Copyright (C) 2004  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
 * GL_PARTICLES Class
 * Encapsulates a image presentation item object
 * -------------------------------------------------------------
 * September, 11th, 2004
 * - File Creation. GNU/EDMA in-line class
 * -------------------------------------------------------------
 * September, 12th, 2004
 * - Added GL_SLOW_ZOOM and GL_FLYING_QUADS effects
 */

/* OpenGL includes */
#include <GL/gl.h>
#include <GL/glu.h>
#include "sdlgl_tex.h"

#include <edma.h>
#include <stdlib.h>
#include <math.h>

#include <time.h>

typedef struct
{
  EReal64   r,g,b;
  EReal64   x_vel, y_vel, z_vel;
  EReal64   x_size, y_size, z_size;
  EReal64   x, y, z;
  EReal64   life;
  EReal64   alpha;
} A_PARTICLE;

typedef struct
{
  ESint32      indx;
  ESint32      n;
  EDMAT_BUFFER quad;
} GL_PARTICLESData;

static SDLGL_TEX  p_tex = NULL;
static GLuint     tex;

ESint32 EDMAPROC GL_PARTICLESborn (OBJID id);
ESint32 EDMAPROC GL_PARTICLESrip (OBJID id);

ESint32 EDMAPROC GL_PARTICLESrun (OBJID id, OBJID *item);
ESint32 EDMAPROC GL_PARTICLESload (OBJID id, OBJID s);
ESint32 EDMAPROC GL_PARTICLESsave (OBJID id, OBJID s);


/************************************************************************************/

ESint32
gl_particles_class_factory ()
{
  CLASSID    cid;

  cid = edma_idf_get_free_class_id (EDMA_LOCAL_CLASS);

  edma_idf_set_class_name (cid, "PARTICLES");
  edma_idf_set_class_namespace (cid, "PRISCILA");
  edma_idf_set_class_version (cid, 0, 0);

  /* Item position in 3D space*/
  edma_add_local_class_property (cid, "indx", DT_EOBJECT, E_L, 0);
  edma_add_local_class_property (cid, "num", DT_ESINT32, E_L, 0);
  edma_add_local_class_property (cid, "data", DT_EBUFFER, E_L, 0);

  /* GL_ITEM methods */
  edma_add_local_class_method (cid, "born", "", (PPROC) GL_PARTICLESborn, 0, 0, 0);
  edma_add_local_class_method (cid, "rip", "", (PPROC) GL_PARTICLESrip, 0, 0, 0);

  edma_add_local_class_method (cid, "run", "", (PPROC) GL_PARTICLESrun, 0, 0, 0);
  edma_add_local_class_method (cid, "save", "O", (PPROC) GL_PARTICLESsave, 0, 0, 0);
  edma_add_local_class_method (cid, "load", "O", (PPROC) GL_PARTICLESload, 0, 0, 0);

  edma_idf_set_class_id (cid);
  return 0;
}


ESint32 EDMAPROC 
GL_PARTICLESborn (OBJID obj)
{
  GL_PARTICLESData   *m = (GL_PARTICLESData*) edma_get_data_ref (obj);
  A_PARTICLE         *part;
  int                i;
  double             velocity, alpha, z, size, asize;


  p_tex = SDLGL_new_texture ();
  edma_printf ("Load result : %d", SDLGL_read_from_file (p_tex, "./particle.jpg"));
  SDLGL_tex_set_scale (p_tex, 1.0f);

  srand (time(NULL));

  velocity = 0.01f;
  alpha = 0.3f;
  z = -20.0;
  size = 0.35f;
  //size = 1.5f;

  edma_printf ("%s", "*** Initialising Particle system ");
  m->n = 1000;
  /* Allocate particle buffer */
  if ((edma_buffer_alloc (&m->quad, sizeof(A_PARTICLE) * m->n)) == -1)
    {
      edma_printf_err ("%s", "Cannot allocate memory for particles");
      return -1;
    }

  part = (A_PARTICLE *) m->quad.dat;

  for (i = 0; i < m->n; i++)
    {
      part[i].r = 1.0f;
      part[i].g = 0.2f;
      part[i].b = 0.2f;
      part[i].x_vel = velocity / 2.0 - ((velocity ) * ((double)rand()/ RAND_MAX));
      part[i].y_vel = velocity + ((velocity ) * ((double)rand()/ RAND_MAX));
      //part[i].y_vel = velocity / 2.0 - ((velocity ) * ((double)rand()/ RAND_MAX));
      part[i].z_vel = 0;

      edma_printf ("Particle %d velocity: %lf, %lf, %lf", i, part[i].x_vel, part[i].y_vel, part[i].z_vel);

      part[i].z   = z;
      part[i].x   = 5 - 10*((double)rand () / RAND_MAX);
      part[i].y   = 0;
      /*
      part[i].x   = 10.0 * (double)rand () / RAND_MAX;
      part[i].y   = y_base + y_delta  * (((double)rand () / RAND_MAX) - 0.5f);
      */
      asize = size * ((double)rand () / (double)RAND_MAX);
      part[i].x_size =  asize;
      part[i].y_size = asize;
      part[i].z_size = asize;
      
      part[i].alpha = alpha * (double)rand() /RAND_MAX;
      //part[i].life = (double)rand() /RAND_MAX;
      part[i].life = 0.5f + 0.5 * (double)rand() /RAND_MAX;
      /*
      if (part[i].sizex < 0.5) part[i].sizex = 0.5;
      if (part[i].sizey < 0.5) part[i].sizey = 0.5;
      */
    }


  return 0;
}

ESint32 EDMAPROC 
GL_PARTICLESrip (OBJID obj)
{
  GL_PARTICLESData   *m = (GL_PARTICLESData*) edma_get_data_ref (obj);

  edma_buffer_free (&m->quad);

  return 0;
}

ESint32 EDMAPROC 
GL_PARTICLESrun (OBJID obj, OBJID *item)
{
  EReal64               x, y, z;
  ESint32               i, j, n, flag;
  A_PARTICLE              *p;
  GL_PARTICLESData   *m = (GL_PARTICLESData*) edma_get_data_ref (obj);

  p = (A_PARTICLE *) m->quad.dat;

  glShadeModel(GL_SMOOTH);	
  glEnable (GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE);

  glDisable (GL_DEPTH_TEST);
  glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);


  n = 5;
  for (i = 0; i < m->n; i++)
    {
      x = p[i].x + p[i].x_vel;
      y = p[i].y + p[i].y_vel;
      z = p[i].z + p[i].z_vel;
      flag = 0;
      for (j = 0; j < n; j++)
	{
	  /* Update Particle Position */
	  p[i].x = p[i].x + p[i].x_vel/n;  /* Move right to left */
	  p[i].y = p[i].y + + p[i].y_vel/n;  /* Move right to left */
	  p[i].z = z;  /* Move right to left */
	  
	  p[i].alpha -= 0.0001f;
	  if (p[i].alpha <= 0.0f) p[i].life = 0;
	  if (p[i].y > 4.0f) p[i].life = 0.30f;
	  if (p[i].y > 8.0f) p[i].life = 0;
	  if (p[i].y < -1.0f) p[i].life = 0;
	  if (flag == 0)
	    {
	      if (p[i].x > 1.0f) 
		{
		  p[i].x_vel -=0.004f;
		}
	      if (p[i].x < -1.0f) 
		{
		  p[i].x_vel +=0.004f;
		}
	    }
	  
	  if (p[i].life < 0.55f && p[i].life > 0.25f)
	    {
	      flag = 1;
	      p[i].b = 0.8f;
	      p[i].r = 0.5f;
	      p[i].g = 0.5f;
	      p[i].y_vel+=0.002f;
	    }

	  if (flag == 0) break;
	  /* Transform Particle */
	  glLoadIdentity();
	  
	  glTranslatef (p[i].x, p[i].y , p[i].z);
	  glScalef (p[i].x_size, p[i].y_size, p[i].z_size);

	  /* Render Particle */
	  //glBindTexture (GL_TEXTURE_2D, tex);
	  //glColor4f (p[i].r, p[i].g, p[i].b, p[i].alpha);
	  glColor4f (p[i].r, p[i].g, p[i].b, p[i].life - p[i].life / n);
	  
	  SDLGL_img_render (p_tex);
	  
	}
      /*
      p[i].y = y;
      p[i].x = x;
      p[i].z = z;
      */
      //p[i].life -= (0.001f * ((double)rand () / RAND_MAX));
      p[i].life -= (0.035f * ((double)rand () / RAND_MAX));
      if (p[i].life <= 0.0f)
	{
	  //p[i].life =((double)rand () / RAND_MAX); 
	  p[i].life = 1.0f;
	  p[i].alpha = 0.5f;
	  p[i].x   = 3 - (6.0*((double)rand () / RAND_MAX));
	  p[i].y = 0.5 +  ((double)rand () / RAND_MAX);
	  p[i].z = -20.0 - 3.0*((double)rand () / RAND_MAX);
	  p[i].y_vel = 0.009f + ((0.009f ) * ((double)rand()/ RAND_MAX));
	  p[i].x_vel = 0.005f - ((0.01f ) * ((double)rand()/ RAND_MAX));
	  p[i].z_vel = 0.005f - ((0.01f ) * ((double)rand()/ RAND_MAX));
	  p[i].r = 1.0f;
	  p[i].g = p[i].b = 0.2f;
	  /* Regenerate particle */
	  continue;
	}
      
    }
  return 0;
}

ESint32 EDMAPROC 
GL_PARTICLESload (OBJID obj, OBJID s)
{
  edma_printf_obj (obj, "%s", "Method still not implemented");
  return 0;
#if 0
  EChar                 buffer[1024];
  EReal64               velocity, z, size, y_base, y_delta, alpha;
  PARTICLE              *part;
  ESint32               len, i;
  GL_FLYING_QUADSData   *m = (GL_FLYING_QUADSData*) edma_get_data_ref (obj);
  
  len = 1024;
  edma_met3 (s, "gets", buffer, &len);
  buffer[strlen(buffer) - 1] = 0; /* Chomp */

  sscanf (buffer, "%d %lf %lf %lf %lf %lf %lf", 
	  (int*)&m->n, &velocity, &size, &z, &y_base, &y_delta, &alpha);

  edma_printf_obj (obj, "NUM: %d | VEL: %lf | SIZE: %lf | Z: %lf Y0:%lf dY:%lf Alpha:%ld", 
		   m->n, velocity, size, z, y_base, y_delta, alpha);

  /* Alloca buffer */
  if ((edma_buffer_alloc (&m->quad, sizeof(PARTICLE) * m->n)) == -1)
    {
      edma_printf_err ("%s", "Cannot allocate memory for particles");
      return -1;
    }

  part = (PARTICLE *) m->quad.dat;

  for (i = 0; i < m->n; i++)
    {
      part[i].vel = velocity * ((double)rand()/ RAND_MAX);
      part[i].z   = z;
      part[i].x   = 10.0 * (double)rand () / RAND_MAX;
      part[i].y   = y_base + y_delta  * (((double)rand () / RAND_MAX) - 0.5f);
      part[i].sizex =  size * ((double)rand () / (double)RAND_MAX);
      part[i].sizey = size * ((double)rand () / (double)RAND_MAX);
      part[i].alpha = alpha * (double)rand() /RAND_MAX;

      if (part[i].sizex < 0.5) part[i].sizex = 0.5;
      if (part[i].sizey < 0.5) part[i].sizey = 0.5;
    }

  /* Particle system initiated */

  return obj;
#endif
}

ESint32 EDMAPROC 
GL_PARTICLESsave (OBJID obj, OBJID s)
{
  GL_PARTICLESData   *m = (GL_PARTICLESData*) edma_get_data_ref (obj);

  edma_printf_obj (obj, "Method 'load' still not implemented");

  return 0;
}
