/*
 * Priscila: A GNU/EDMA OpenGL Presentation Tool
 * Copyright (C) 2004  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
 * GL_STYLES Class
 * Encapsulates a set of text styles for a presentation
 * -------------------------------------------------------------
 * September, 8th, 2004
 * - File Creation
 */
#include <string.h>
#include <edma.h>

/* XXX: This class should also manage fonts, almost when loading 
 *      styles for presentation 
 */

/* Data Structure for GL_STYLES */
typedef struct
{
  EDMAT_BUFFER    style;
  ESint32         num;
} GL_STYLESData;

/* Low Level data structure encapsulated within EDMAT_BUFFER */

typedef struct
{
  EChar     name[256];
  OBJID     font;
  EReal64   scale;
} GL_STYLE;

/* Method prototypes */
ESint32 EDMAPROC GL_STYLESadd (OBJID obj, EPChar name, OBJID font, EReal64 scale);
ESint32 EDMAPROC GL_STYLESdel (OBJID obj, EPChar name);
ESint32 EDMAPROC GL_STYLESget_data (OBJID obj, EPChar name, OBJID *font, EReal64 *scale);

ESint32 EDMAPROC GL_STYLESload (OBJID obj, EPChar filename);
ESint32 EDMAPROC GL_STYLESsave (OBJID obj, EPChar filename);

ESint32 EDMAPROC GL_STYLESborn (OBJID obj);
ESint32 EDMAPROC GL_STYLESrip (OBJID obj);


/* Class Factory*/
ESint32
gl_styles_class_factory ()
{
  CLASSID    cid;

  cid = edma_idf_get_free_class_id (EDMA_LOCAL_CLASS);

  edma_idf_set_class_name (cid, "GL_STYLES");
  edma_idf_set_class_namespace (cid, "PRISCILA");
  edma_idf_set_class_version (cid, 0, 0);

  /* Item position in 3D space*/
  edma_add_local_class_property (cid, "style", DT_EBUFFER, E_L, 0);
  edma_add_local_class_property (cid, "num", DT_ESINT32, E_L, 0);

  /* GL_ITEM methods */
  edma_add_local_class_method (cid, "add", "ZOR64", (PPROC) GL_STYLESadd, 0, 0, 0);
  edma_add_local_class_method (cid, "del", "Z", (PPROC) GL_STYLESdel, 0, 0, 0);
  edma_add_local_class_method (cid, "get_data", "ZsOsR64", (PPROC) GL_STYLESget_data, 0, 0, 0);

  edma_add_local_class_method (cid, "load", "Z", (PPROC) GL_STYLESload, 0, 0, 0);
  edma_add_local_class_method (cid, "save", "Z", (PPROC) GL_STYLESsave, 0, 0, 0);

  /* Constructor/destructor*/
  edma_add_local_class_method (cid, "born", "", (PPROC) GL_STYLESborn, 0, 0, 0);
  edma_add_local_class_method (cid, "rip", "", (PPROC) GL_STYLESrip, 0, 0, 0);

  edma_idf_set_class_id (cid);

  return 0;
}

ESint32 EDMAPROC 
GL_STYLESadd (OBJID obj, EPChar name, OBJID font, EReal64 scale)
{
  GL_STYLE       *aux;
  GL_STYLESData  *m = edma_get_data_ref (obj);

  if ((edma_buffer_realloc (&m->style, sizeof (GL_STYLE) * (m->num + 1))) < 0)
    {
      edma_printf_obj (obj, "Cannot allocate memory to add style %s", name);
      return -1;
    }
  aux = (GL_STYLE *) m->style.dat;
  
  strncpy (aux[m->num].name, name, 256);
  aux[m->num].font = font;
  aux[m->num].scale = scale;

  m->num++;

  return 0;
}

ESint32 EDMAPROC 
GL_STYLESdel (OBJID obj, EPChar name)
{
  GL_STYLESData  *m = edma_get_data_ref (obj);

  edma_printf_obj (obj, "%s", "Method 'del' still not implemented");

  return 0;
}


ESint32 EDMAPROC 
GL_STYLESget_data (OBJID obj, EPChar name, OBJID *font, EReal64 *scale)
{
  ESint32        i;
  GL_STYLE       *aux;
  GL_STYLESData  *m = edma_get_data_ref (obj);

  aux = (GL_STYLE*) m->style.dat;

  for (i = 0; i < m->num; i++)
    if ((strncmp (name, aux[i].name, 256)) == 0)
      break;

  if (i  == m->num)
    {
      *font = -1;
      *scale = 1.0;
      return -1;
    }
  else
    {
      *font = aux[i].font;
      *scale = aux[i].scale;
      return i;
    }
}

ESint32 EDMAPROC 
GL_STYLESload (OBJID obj, EPChar filename)
{
  OBJID          s;
  EChar          name[256];
  EChar          font[256];
  float          size;
  ESint32        len;
  GL_STYLESData  *m = edma_get_data_ref (obj);

  if ((s = edma_new_obj ("NFILE")) == -1)
    {
      edma_printf_err ("%s", "STYLES::load -> Cannot create NFILE object.");
      return -1;
    }

  if ((edma_met3 (s, "open", filename, "rt")) == -1)
    {
      edma_printf_err ("STYLES::load -> Cannot open file '%s'", filename);
      return -1;
    }

  

  edma_met3 (s, "close");

  return 0;
}

ESint32 EDMAPROC 
GL_STYLESsave (OBJID obj, EPChar filename)
{
  GL_STYLESData  *m = edma_get_data_ref (obj);

  edma_printf_obj (obj, "%s", "Method 'save' still not implemented");

  return 0;
}

ESint32 EDMAPROC
GL_STYLESborn (OBJID obj)
{
  GL_STYLESData  *m = edma_get_data_ref (obj);

  m->num = 0;

  return 0;
}

/* Destructor frees GL_FONT objects contained */
ESint32 EDMAPROC 
GL_STYLESrip (OBJID obj)
{
#if 0
  int            i;
  GL_STYLE       *aux;
#endif 
  GL_STYLESData  *m = edma_get_data_ref (obj);

#if 0
  aux = (GL_STYLE*) m->style.dat;

  for (i = 0; i < m->num; i++)
    edma_free_obj (aux[i].font);
#endif 
  edma_buffer_free (&m->style);
  return 0;
}


