/*
 * Priscila: A GNU/EDMA OpenGL Presentation Tool
 * Copyright (C) 2004  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 
/*
 * Simple test of GNU/EDMA GL_ITEM hierarchy
 * ------------------------------------------------------------
 * September, 10th, 2004
 * - Introduced GL_SLIDE class
 */

#include <GL/gl.h>
#include <GL/glu.h>

#include <SDL.h>
#include <SDL_ttf.h>

#include "sdlgl_init.h"

#include <stdio.h>
#include <strings.h>
#include <stdlib.h>

#include <edma.h>

#include "gl_item.h"
#include "gl_item_text.h"
#include "gl_item_image.h"
#include "gl_item_3ds.h"
#include "gl_styles.h"
#include "gl_slide.h"
#include "gl_particles.h"

#include "gl_fx.h"

/* Constants */
#define MAX_FILE_NAME 255
#define MAX_FONTS     20

/* Global Variables */
OBJID   font[MAX_FONTS];      /* Array of used TT Fonts in slide */
OBJID   slide;                /* Current slide to be draw */
OBJID   particles;

void 
DrawGLScene()
{
  //ESint32   i;

  /***** Render the rotating Board *********/
  /* Clear the screen and the depth buffer*/
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  //edma_met3 (particles, "run");

  edma_met3 (slide, "render");



  glEnable (GL_DEPTH_TEST);
  glDisable (GL_BLEND);

  SDL_GL_SwapBuffers();
}

int
main (int argc, char *argv[])
{
  ESint32     done, i;
  OBJID       styles;

  edma_printf ("Priscila Version %s. A GNU/EDMA OpenGL Presentation Tool", VERSION);
  edma_printf ("(c) David Martnez Oliveira, 2004");
  edma_printf ("This is free software; see the source for copying conditions");

  if (argc != 2)
    {
      edma_printf_err ("%s", "Wrong number of parameters");
      edma_printf_err ("%s", "Use: priscila slide.pris");
      exit (1);
    }

  EDMAInit();

  /* Define GNU/EDMA Local Classes */

  gl_styles_class_factory ();
  gl_font_class_factory ();
  gl_slide_class_factory ();

  gl_item_class_factory ();  
  gl_item_free_text_class_factory ();
  gl_item_text_class_factory ();
  gl_item_image_class_factory ();
  gl_item_quad_class_factory ();
  gl_item_3ds_class_factory ();

  gl_fx_slow_zoom_class_factory ();
  gl_flying_quads_class_factory ();
  gl_spin_object_class_factory ();
  gl_anim_object_class_factory ();

  gl_particles_class_factory ();



  for (i = 0; i < MAX_FONTS; i++) font[i] = -1;

  /* Initialice Video mode */
  sdlgl_init_video (1024, 768);
  //particles = edma_new_obj ("PARTICLES"); 
  SDL_EnableKeyRepeat (SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);

  /* Create Styles Array object */
  if ((styles = edma_new_obj ("GL_STYLES")) == -1)
    {
      edma_printf_err ("%s", "Cannot create object of class GL_STYLES... Aborting");
      EDMAEnd ();
      return -1;
    }

  /* Create Single Font */
  /* Must be added to GL_STYLE singleton */
  font[0] = edma_new_obj ("GL_FONT");
  edma_met3 (font[0], "build", "verdanab.ttf", 64);
  edma_met3 (font[0], "set_fgcolor", 0.8, 0.8, 0.9);

  font[1] = edma_new_obj ("GL_FONT");
  edma_met3 (font[1], "build", "verdana.ttf", 32);
  edma_met3 (font[1], "set_fgcolor", 0.8, 0.8, 0.8);

  font[2] = edma_new_obj ("GL_FONT");
  edma_met3 (font[2], "build", "helveticab.ttf", 32);
  edma_met3 (font[2], "set_fgcolor", 0.8, 0.8, 0.8);

  /* Add presentation styles */
  edma_met3 (styles, "add", "TITLE", font[0], 0.8f);
  edma_met3 (styles, "add", "NORMAL", font[1], 0.5f);
  edma_met3 (styles, "add", "SMALL", font[1], 0.3f);
  edma_met3 (styles, "add", "NORMAL1", font[2], 0.5f);
  edma_met3 (styles, "add", "SMALL1", font[2], 0.3f);

  /* Create singleton for global access to styles */
  edma_smet3 ("SINGLETON", "Register", "STYLES", styles);

  /* Create slide object */
  if ((slide = edma_new_obj ("GL_SLIDE")) == -1)
    {
      edma_printf_err ("%s", "Cannot create object of class 'GL_SLIDE'");
    }
  
  edma_met3 (slide, "load", argv[1]);

  done = 0;
  /* Main loop */
  while (!done)
    {
      DrawGLScene ();
      {
	SDL_Event    event;
	while (SDL_PollEvent (&event))
	  {
	    /* Process main QUIT event */
	    if (event.type == SDL_QUIT)
	      {
		done = 1;
		break;
	      }
	    if (event.type == SDL_KEYDOWN)
	      done = edma_met3 (slide, "process", &event);
	  }
      }
      SDL_Delay (50); /* Free CPU */
    }



  /* Free Data */
  for (i = 0; i < MAX_FONTS && font[i] != -1; i++) edma_free_obj (font[i]);

  edma_free_obj (slide);
  edma_free_obj (styles);
    
  EDMAEnd();

  SDL_Quit();

  return 0;
}


