
#include <GL/gl.h>
#include <GL/glu.h>

#include <SDL.h>
#include <SDL_ttf.h>

/* Lights */
/* white ambient light at half intensity (rgba) */
GLfloat LightAmbient[] = { 0.01f, 0.01f, 0.01f, 0.9f };

/* super bright, full intensity diffuse light. */
//GLfloat LightDiffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat LightDiffuse[] = { 0.01f, 0.01f, 0.01f, 0.9f };

/* position of light (x, y, z, (position of light)) */
//GLfloat LightPosition[] = { 0.0f, 0.0f, 2.0f, 1.0f };
GLfloat LightPosition[] = { 0.0f, 0.0f, 2.0f, 0.0f };




/* A general OpenGL initialization function.  
 * Sets all of the initial parameters. 
 * from NeHe website 
 * Modified 
 */
GLvoid 
InitGL(GLsizei Width, GLsizei Height)
{
  glViewport (0, 0, Width, Height);
  
  /* Load textures*/
  glEnable (GL_TEXTURE_2D);              /* Enable texture mapping.*/
  
  glClearColor (0.0f, 0.0f, 0.0f, 0.0f); /* This Will Clear The Background Color To Black*/
  glClearDepth (1.0);			 /* Enables Clearing Of The Depth Buffer */
  //glDepthFunc (GL_LESS);		 /* The Type Of Depth Test To Do */
  glDepthFunc (GL_LEQUAL);		 /* The Type Of Depth Test To Do */
  glEnable (GL_DEPTH_TEST);		 /* Enables Depth Testing */
  glShadeModel (GL_SMOOTH);		 /* Enables Smooth Color Shading */
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();			 /* Reset The Projection Matrix */
  
  /* Calculate The Aspect Ratio Of The Window */
  gluPerspective (45.0f, (GLfloat)Width / (GLfloat)Height, 0.1f, 100.0f);	
  
  glMatrixMode(GL_MODELVIEW);
  
  /* set up light number 1. */
  glLightfv (GL_LIGHT1, GL_AMBIENT,  LightAmbient);  /* add lighting. (ambient) */
  glLightfv (GL_LIGHT1, GL_DIFFUSE,  LightDiffuse);  /* add lighting. (diffuse). */
  glLightfv (GL_LIGHT1, GL_POSITION, LightPosition); /* set light position. */
  glEnable( GL_LIGHT1);                             /* turn light 1 on. */
  
  /* setup blending */
  glBlendFunc (GL_SRC_ALPHA, GL_ONE);	/* Set The Blending Function For Translucency*/
  glColor4f(1.0f, 1.0f, 1.0f, 0.5);    
  
#if 0
  /* Init for Quadrics.... I don't realy know what this does */
  /* Required to render spheres */
  /* Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW) */
  quadratic = gluNewQuadric ();			
  gluQuadricNormals (quadratic, GLU_SMOOTH);	/* Create Smooth Normals (NEW) */
  gluQuadricTexture (quadratic, GL_TRUE);	/* Create Texture Coords (NEW) */
#endif
}

int 
sdlgl_init_video (int w, int h)
{
  /* Initialize SDL for video output */
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }
  
  /* Create a OpenGL screen */
  if ( SDL_SetVideoMode(w, h, 0, SDL_OPENGL) == NULL ) {
    fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
    SDL_Quit();
    exit(2);
  }
  
  /* Set the title bar in environments that support it */
  SDL_WM_SetCaption("SDLGL application", NULL);
  
  /* Loop, drawing and checking events */
  InitGL(w, h);
  
  TTF_Init ();

  return 0;
}

int 
sdlgl_quit_video ()
{
  SDL_Quit();
  return 0;
}
