#include <stdio.h>
#include <stdlib.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include <SDL.h>
#include <SDL_ttf.h>
#include <SDL_image.h>

#include "sdlgl_font.h"

#define SDLGL_LISTBOX_C

typedef struct sdlgl_listbox_t
{
  SDLGL_FONT    *font;
  SDLGL_TXT     *option;
  int           n_options;
  int           current_option;
  int           target_option;
  int           visible_options;
  int           first_option;
  GLfloat       current_pos;
  GLfloat       target_pos;
  GLfloat       inc_pos;
  GLfloat       width_scale;
  int           moving;
  GLfloat       vel;
} *SDLGL_LISTBOX;

#include "sdlgl_listbox.h"

SDLGL_LISTBOX
SDLGL_listbox_new (SDLGL_FONT font, int v_op)
{
  SDLGL_LISTBOX  the_list;

  printf ("Creating LISTBOX storage request %ld\n",
	  sizeof (struct sdlgl_listbox_t));
  if ((the_list = (SDLGL_LISTBOX) malloc (sizeof (struct sdlgl_listbox_t))) == NULL)
    {
      fprintf (stderr, "Can't alloc memory for new LISTBOX object\n");
      return NULL;
    }
  memset (the_list, 0, sizeof (struct sdlgl_listbox_t));
  the_list->vel = 10.0f;
  the_list->font = font;
  the_list->visible_options = v_op;
  return the_list;
}

int
SDLGL_listbox_free (SDLGL_LISTBOX listbox)
{
  int    i;

  if (listbox == NULL)
    return -1;
  for (i = 0; i < listbox->n_options; i++)
    SDLGL_free_texture (listbox->option[i]);

  free (listbox->option);
  free (listbox);
  return 0;
}

int
SDLGL_listbox_add (SDLGL_LISTBOX listbox, char *text)
{
  GLfloat    scale;

  if (listbox == NULL)
    return -1;
  if ((listbox->option = 
       (SDLGL_TEX*) realloc (listbox->option, (listbox->n_options + 1) * sizeof (SDLGL_TEX*)))
      == NULL)
    {
      fprintf (stderr, "Can't add new option to current PRIMS_MENU\n");
      return -1;
    }
  listbox->option[listbox->n_options] = 
    SDLGL_txt_new (listbox->font, text, 0.5f);

  scale = SDLGL_txt_get_aspect (listbox->option[listbox->n_options]);
  if (scale > listbox->width_scale)
    listbox->width_scale = scale + 2.0f;

  /* FIXME: check error and roll back*/
  listbox->n_options++;
  return 0;
}

int
SDLGL_listbox_get_option (SDLGL_LISTBOX listbox)
{
  return listbox->target_option;
}

int
SDLGL_listbox_get_moving (SDLGL_LISTBOX listbox)
{
  return listbox->moving;
}

int
SDLGL_listbox_set_option (SDLGL_LISTBOX listbox, int option)
{
  int    direction;


  if (listbox == NULL)
    return -1;
  if (listbox->current_option == option)
    return 0;
  
  if (listbox->current_option != listbox->target_option)
    return 0;

  if (option <=0)
    option = 0;
  if (option >= listbox->n_options)
    option = listbox->n_options - 1;
  listbox->current_option = option;
  listbox->target_option = option;

  /* Calculate first option*/
  if (option < listbox->first_option)
    listbox->first_option = option;
  else
    {
      if (option > (listbox->first_option + listbox->visible_options - 1))
	listbox->first_option++;
    }

  printf ("[%s] Current option: %d trying to set: %d\n", __FUNCTION__, listbox->current_option, option);
#if 0
  

  listbox->target_option = option;
  if (listbox->target_option > listbox->current_option)
    direction = 1;
  else
    direction = 0;
  if (option < 0)
    {
      listbox->target_option = listbox->n_options - 1;
    }
  if (option >= listbox->n_options)
    {
      listbox->target_option = option % listbox->target_option;
    }

  printf ("Current option: %d Target Option: %d\n", listbox->current_option, listbox->target_option);
  //listbox->target_option = option;
  /* If we move to the right */
  //if (listbox->target_option > listbox->current_option)
  if (direction)
    {
      listbox->target_angle = listbox->current_angle - 90.0f;
      //listbox->inc_angle = -10.0f;
      listbox->inc_angle = -listbox->vel;
    }
  else
    {
      listbox->target_angle = listbox->current_angle + 90.0f;
      //listbox->inc_angle = 10.0f;
      listbox->inc_angle = listbox->vel;
    }
#endif
  return listbox->current_option;
}

int
SDLGL_listbox_render (SDLGL_LISTBOX listbox)
{
  int       option, prev, next;
  int       n, i;
  GLfloat   y_pos;
  SDLGL_TEX current;
  GLfloat   x0, y0, x1, y1, scale;
  GLfloat   texc[4];
  GLfloat   texMinX, texMinY, texMaxX, texMaxY;

  if (listbox == NULL)
    return -1;

  n = (listbox->n_options > listbox->visible_options ?
       listbox->visible_options : listbox->n_options);

  glPushMatrix();
  glPushMatrix();

  for (i = 0; i < n; i++)
    {
      //glLoadIdentity();
      glPopMatrix();
      glPushMatrix();
      SDLGL_txt_simple_render (listbox->option[listbox->first_option + i]);
      glPopMatrix();
      glTranslatef (0.0f, -0.5f - 0.4f, 0.0f); /* Move down to next line*/
      glPushMatrix();
      //glPushMatrix();
    }

  glPopMatrix();
  glPopMatrix();

  glBlendFunc (GL_SRC_ALPHA, GL_ONE);
  glColor4f (1.0f, 1.0f, 1.0f, 0.5);
  glEnable (GL_BLEND);
  glDisable (GL_DEPTH_TEST);

  glPushMatrix();
  glTranslatef (0.0f, (-0.5f -0.4f) * 
		(listbox->current_option - listbox->first_option), -0.09f);
  glScalef (listbox->width_scale / 2, 0.5f, 0.5f);


  glBindTexture (GL_TEXTURE_2D, 0);
  glBegin(GL_QUADS);

  glNormal3f( 0.0f, 0.0f, 0.1f); /* Normal is Up*/
  //glColor4f ( 0.5f, 0.5f, 1.0f, 0.5f);
  glColor4f ( 0.2f, 0.2f, 0.8f, 0.99f);
  glVertex3f(  1.0f,  1.0f,  0.0f);
  glVertex3f(  1.0f, -1.0f,  0.0f);
  glVertex3f( -1.0f, -1.0f,  0.0f);
  glVertex3f( -1.0f, 1.0f,  0.0f);
  
  glEnd();
  glColor4f (1.0f, 1.0f, 1.0f, 0.5);
  glPopMatrix();

return 0;
}
