/*
 * Helper functions for OpenGL & SDL
 */

#include <stdlib.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include <SDL.h>
#include <SDL_ttf.h>
#include <SDL_image.h>

typedef struct sdlgl_texture_t
{
  GLuint      tex;
  SDL_Surface *surf;
  GLfloat     texcoord[4];
  GLfloat     scale;
  int         w, h;
  GLfloat     nx, ny, nz;
} *SDLGL_TEX;


#define SDLGL_C
#include "sdlgl_tex.h"
GLuint _SDL_GL_LoadTexture(SDL_Surface *surface, GLfloat *texcoord);
/* Internal datatypes */

SDLGL_TEX
SDLGL_new_texture ()
{
  SDLGL_TEX   the_tex;

  if ((the_tex = (SDLGL_TEX)malloc (sizeof (struct sdlgl_texture_t))) == NULL)
    {
      fprintf (stderr, "%s", "Can't alloc memory for SDLGL_Texture\n");
      return NULL;
    }
  memset (the_tex, 0, sizeof (struct sdlgl_texture_t));
  the_tex->tex = -1;
  return the_tex;
}

int
SDLGL_free_texture(SDLGL_TEX tex)
{
  
  /* Free GL texture... I still don't know what is the function :P*/
  if (tex->tex != -1)
    {
      glDeleteTextures (1, &tex->tex);
      tex->tex = -1;
    }
  if (tex->surf)
    {
      SDL_FreeSurface (tex->surf);
      tex->surf = NULL;
    }
    
  free (tex);
  return 0;
}

/******************************************************************/
/* ACCESSOR Functions */
/******************************************************************/
GLuint
SDLGL_tex_get_gltex (SDLGL_TEX tex)
{
  return tex->tex;
}

SDL_Surface*
SDLGL_tex_get_surface (SDLGL_TEX tex)
{
  return tex->surf;
}

int
SDLGL_tex_get_size (SDLGL_TEX tex, int *w, int *h)
{
  *w = tex->surf->w;
  *h = tex->surf->h;
  return 0;
}

int
SDLGL_tex_get_map_coord (SDLGL_TEX tex, GLfloat *coord)
{
  coord[0] = tex->texcoord[0];
  coord[1] = tex->texcoord[1];
  coord[2] = tex->texcoord[2];
  coord[3] = tex->texcoord[3];
  return 0;
}

int
SDLGL_tex_set_scale (SDLGL_TEX tex, GLfloat scale)
{
  tex->scale = scale;
  return 0;
}

int 
SDLGL_tex_set_normal (SDLGL_TEX tex, GLfloat x, GLfloat y, GLfloat z)
{
  tex->nx = x;
  tex->ny = y;
  tex->nz = z;
  
  return 0;
}
/***********************************************************************/
int
SDLGL_read_from_file (SDLGL_TEX tex, char *file_name)
{
  
  tex->surf = IMG_Load (file_name);
  /* FIXME: See the documentation to deal with errors in SDL_Image */
  tex->tex = _SDL_GL_LoadTexture (tex->surf, tex->texcoord);

  /* FIXME: More error checking */
  return 0;
}

int
SDLGL_read_from_surface (SDLGL_TEX tex, SDL_Surface *surf)
{
  if (!surf)
    {
      fprintf (stderr, "Surface is NULL\n");
      return -1;
    }
  
  tex->surf = surf;
  /* FIXME: See the documentation to deal with errors in SDL_Image */
  tex->tex = _SDL_GL_LoadTexture (tex->surf, tex->texcoord);

  /* FIXME: More error checking */
  return 0;
}

int 
SDLGL_img_render (SDLGL_TEX tex)
{
  GLfloat   texMinX, texMinY, texMaxX, texMaxY;
  GLfloat   x,y, x0, y0, x1, y1;
  int       w, h;

  w = tex->surf->w;
  h = tex->surf->h;
  x = y = 0;
  texMinX = tex->texcoord[0];
  texMinY = tex->texcoord[1];
  texMaxX = tex->texcoord[2];
  texMaxY = tex->texcoord[3];

  glBindTexture (GL_TEXTURE_2D, tex->tex);
  //img_scale = 9;

  //texMaxX = 1.0f;
#if 0
  if (w > h)
    glTranslatef (0.0f, 6.0f, z);
  else
    glTranslatef (5.5f, 0.0f, z);
#endif
#if 1
  /* Correct aspect ration for the projection model 3x4 */
  if (w > h)
    glScalef (tex->scale, tex->scale * (3.0f / 4.0f), tex->scale);
  else
    glScalef (tex->scale * (3.0f / 4.0f), tex->scale, tex->scale);
#endif
  //glScalef (tex->scale * (3.0f / 4.0f), tex->scale, tex->scale);
  //glScalef (tex->scale, tex->scale, tex->scale);
  /* Calculate the QUADS vertex for the current image size */
  x0 = -1.0f;
  y0 = -1.0f;
#if 0
  if (w < h)
    {
      x1 = -1.0f + texMaxX / 2.0f;
      y1 = 1.0f;
    }
  else
    {
      y1 = 1.0f - texMaxY / 2.0f;
      x1 = 1.0f;
    }
#endif
  x1=y1=1.0f;
  /* Render the QUAD with the image as texture*/
  glBegin(GL_QUADS);

  //glNormal3f( 0.0f, 0.0f, 1.0f); /* Normal is Up*/	
  glNormal3f( tex->nx, tex->ny, tex->nz);
#if 0
  glTexCoord2f( texMinX,  texMinY); glVertex2i( x0,  y1);
  glTexCoord2f( texMaxX,  texMinY); glVertex2i( x1,  y1);
  glTexCoord2f( texMaxX,  texMaxY); glVertex2i( x1,  y0);
  glTexCoord2f( texMinX,  texMaxY); glVertex2i( x0,  y0);
#else
  
  glTexCoord2f( texMinX,  texMinY); glVertex3f( x0,  y1, 0);
  glTexCoord2f( texMaxX,  texMinY); glVertex3f( x1,  y1, 0);
  glTexCoord2f( texMaxX,  texMaxY); glVertex3f( x1,  y0, 0);
  glTexCoord2f( texMinX,  texMaxY); glVertex3f( x0,  y0, 0);

 

#endif
  glEnd();

  return 0;
}


int 
SDLGL_img_prepare_render (SDLGL_TEX tex, GLfloat *x0, GLfloat *y0, GLfloat *x1, GLfloat *y1)
{
  GLfloat   texMinX, texMinY, texMaxX, texMaxY;
  //GLfloat   x,y, x0, y0, x1, y1;
  int       w, h;

  w = tex->surf->w;
  h = tex->surf->h;
  texMinX = tex->texcoord[0];
  texMinY = tex->texcoord[1];
  texMaxX = tex->texcoord[2];
  texMaxY = tex->texcoord[3];
  
  glBindTexture (GL_TEXTURE_2D, tex->tex);
  //img_scale = 9;

  texMaxX = 1.0f;
#if 0
  if (w > h)
    glTranslatef (0.0f, 6.0f, z);
  else
    glTranslatef (5.5f, 0.0f, z);
#endif

#if 0
  /* Correct aspect ration for the projection model 3x4 */
  if (w > h)
    glScalef (tex->scale, tex->scale * (3.0f / 4.0f), tex->scale);
  else
    glScalef (tex->scale * (3.0f / 4.0f), tex->scale, tex->scale);

#endif
  //glScalef (tex->scale * (3.0f / 4.0f), tex->scale, tex->scale);
  /* Calculate the QUADS vertex for the current image size */
  *x0 = -1.0f;
  *y0 = -1.0f;
  if (w > h)
    {
      *x1 = -1.0f + texMaxX / 2.0f;
      *y1 = 1.0f;
    }
  else
    {
      *y1 = 1.0f - texMaxY / 2.0f;
      *x1 = 1.0f;
    }

  *x1=*y1=1.0f;

  return 0;
}


/*************************************************************************/
/*
 * COmplete the API
 * - Create from memory data
 *************************************************************************/

/***************************************************************************/
/********** SUPPORT FUNCTIONS                        ***********************/
/***************************************************************************/

/* Quick utility function for texture creation */
static int 
_power_of_two(int input)
{
  int value = 1;
  
  while ( value < input ) {
    value <<= 1;
  }
  return value;
}

GLuint 
_SDL_GL_LoadTexture(SDL_Surface *surface, GLfloat *texcoord)
{
  GLuint texture;
  int w, h, i, j;
  SDL_Surface *image;
  SDL_Rect area;
  Uint32 saved_flags;
  Uint8  saved_alpha;
  
  /* Use the surface width and height expanded to powers of 2 */
  w = _power_of_two(surface->w);
  h = _power_of_two(surface->h);

  printf ("[%s] Rounded size: %d, %d  Real size: %d, %d\n", 
	  __FUNCTION__, w, h, surface->w, surface->h);

  texcoord[0] = 0.0f;			/* Min X */
  texcoord[1] = 0.0f;			/* Min Y */
  texcoord[2] = (GLfloat)surface->w / w;	/* Max X */
  texcoord[3] = (GLfloat)surface->h / h;	/* Max Y */

  printf ("[%s] Texture Mapping Coordinates: (%f,%f) - (%f,%f)\n",
	  __FUNCTION__, texcoord[0], texcoord[1], texcoord[2], texcoord[3]);

  image = SDL_CreateRGBSurface(
			       SDL_SWSURFACE,
			       w, h,
			       32,
#if SDL_BYTEORDER == SDL_LIL_ENDIAN /* OpenGL RGBA masks */
			       0x000000FF, 
			       0x0000FF00, 
			       0x00FF0000, 
			       0xFF000000
#else
			       0xFF000000,
			       0x00FF0000, 
			       0x0000FF00, 
			       0x000000FF
#endif
			       );
  if ( image == NULL ) {
    return 0;
  }
  
  /* Save the alpha blending attributes */
  saved_flags = surface->flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
  saved_alpha = surface->format->alpha;
  if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
    SDL_SetAlpha(surface, 0, 0);

  }

  /* Copy the surface into the GL texture image */
  area.x = 0;
  area.y = 0;
  area.w = surface->w;
  area.h = surface->h;
  SDL_BlitSurface(surface, &area, image, &area);

  
  /* Restore the alpha blending attributes */
  if ( (saved_flags & SDL_SRCALPHA) == SDL_SRCALPHA ) {
    SDL_SetAlpha(surface, saved_flags, saved_alpha);
  }

  
  /* Create an OpenGL texture for the image */
  glGenTextures(1, &texture);
  glBindTexture(GL_TEXTURE_2D, texture);
  /*
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  */
  /**** Changed to render with LINEAR interpolation **********/

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

  glTexImage2D(GL_TEXTURE_2D,
	       0,
	       GL_RGBA,
	       w, h,
	       0,
	       GL_RGBA,
	       GL_UNSIGNED_BYTE,
	       image->pixels);
  
  //gluBuild2DMipmaps(GL_TEXTURE_2D, 4, w, h, GL_RGBA, GL_UNSIGNED_BYTE, image->pixels);

  SDL_FreeSurface(image); /* No longer needed */
  
  return texture;
}
